# $Id: Loandesk.pm 31596 2010-07-13 11:24:41Z wsl $
# $URL: https://infix.uvt.nl/its-id/trunk/sources/loandesk/lib/UvT/Loandesk.pm $

use utf8;
use strict;
use warnings FATAL => 'all';

package UvT::Loandesk;

use UvT::Loandesk::Mainpage;
use UvT::Loandesk::Reserve;
use UvT::Loandesk::Order;

use Xyzzy -Base;

field 'secret', -init => 'die "no LoandeskSecret configured\n"';
field 'mps', -init => 'die "no LoandeskMPS configured\n"';
field 'app_id', -init => 'die "no LoandeskAselectID configured\n"';
field 'nonce_expiry' => 300;

field 'ldap_server' => 'ldap://localhost';
field 'ldap_base', -init => 'die "no LDAPBase configured\n"';
field 'ldap_filter';
field 'ldap_attribute' => 'uid';

field 'authbroker_expiry' => 43200;
field 'authbroker_host', -init => 'die "no AuthBrokerHost configured\n"';
field 'authbroker_port' => 4033;
field 'authbroker_timeout' => 10;

*set_loandesksecret = *secret;
*set_loandeskmps = *mps;
*set_loandeskaselectid = *app_id;

*set_ldapserver = *ldap_server;
*set_ldapbase = *ldap_base;
*set_ldapfilter = *ldap_filter;
*set_ldapattribute = *ldap_attribute;

*set_authbrokerexpiry = *authbroker_expiry;
*set_authbrokerhost = *authbroker_host;
*set_authbrokerport = *authbroker_port;
*set_authbrokertimeout = *authbroker_timeout;

our %units = (
	'' => 1,
	s => 1,
	m => 60,
	h => 3600,
	d => 86400,
	w => 604800,
	m => 31 * 86400,
	q => 92 * 86400,
	y => int(365.2425 * 86400),
);

sub parse_time {
	my ($time) = @_;
	$time =~ /^(\d+)\s*([a-z]?)$/i;
	my ($seconds, $unit) = ($1, $2);
	die "can't parse time value '$time'"
		unless defined $seconds;
	$unit = $units{lc $unit};
	die "unknown unit $unit\n"
		unless defined $unit;
	return $seconds * $unit;
}

sub set_loandesknonceexpiry {
	eval { $self->nonce_expiry($self->parse_time(@_)) };
	warn "LoandeskExpiry: $@" if $@;
}

sub set_authbrokerexpiry {
	eval { $self->authbroker_expiry($self->parse_time(@_)) };
	warn "AuthBrokerExpiry: $@" if $@;
}

sub set_authbrokertimeout {
	eval { $self->authbroker_timeout($self->parse_time(@_)) };
	warn "AuthBrokerTimeout: $@" if $@;
}

our %generators = (
	'' => 'UvT::Loandesk::Mainpage',
	'reserve' => 'UvT::Loandesk::Reserve',
	'order' => 'UvT::Loandesk::Order',
);

sub generators { \%generators }
