# $Id: LDAP.pm 31543 2010-07-08 15:21:34Z wsl $
# $URL: https://infix.uvt.nl/its-id/trunk/sources/loandesk/lib/UvT/Loandesk/LDAP.pm $

use utf8;
use strict;
use warnings FATAL => 'all';

package UvT::Loandesk::LDAP;

use Spiffy -Base;

use Net::LDAP;
use Net::LDAP::Filter;

sub ldap_connection {
	my $cfg = $self->cfg;

	# create a, possible cached, LDAP client object
	our %ldaps;
	my $server = $cfg->ldap_server;
	my $ldap = $ldaps{$server};
	if(defined $ldap) {
		eval { $ldap->bind };
		if($@) {
			warn "LDAP server $server: $@"
				unless $@ =~ /^Unexpected EOF /;
			undef $ldap;
			delete $ldaps{$server};
		}
	}
	unless(defined $ldap) {
		$ldap = new Net::LDAP($server, onerror => 'die', timeout => 10)
			or die "Connecting to $server: $!";
		$ldap->bind;
		$ldaps{$server} = $ldap;
	}

	return $ldap;
}

sub ldap_search {
	my ($ldap, $uid, $attrs) = @_;

	return undef unless defined $uid;

	my $cfg = $self->cfg;
	my $search;

	# attribute to search on
	my $attribute = $cfg->ldap_attribute;
	my $filterstruct = {equalityMatch => {attributeDesc => $attribute, assertionValue => $uid}};

	# create a, possible cached, filter
	my $filter = $cfg->ldap_filter;
	if(defined $filter) {
		my $given;
		our %filters;
		if(exists $filters{$filter}) {
			$given = $filters{$filter};
		} else {
			eval { $given = new Net::LDAP::Filter($filter); };
			die "Error parsing $filter: $@" if $@;
			$filters{$filter} = $given;
		}
		$search = bless({and => [$given, $filterstruct]}, 'Net::LDAP::Filter')->as_string;
	} else {
		$search = bless($filterstruct, 'Net::LDAP::Filter')->as_string;
	}

	my $base = $cfg->ldap_base;

	my $res = $ldap->search(
			base => $base,
			filter => $search,
			attrs => $attrs
		);

	return ($ldap, $res);
}

sub ldap_anr {
	my ($uid, $passwd) = @_;

	my $ldap = $self->ldap_connection;

	my $res = $self->ldap_search($ldap, $uid, ['employeeNumber']);

	my $anr;
	foreach my $entry ($res->entries) {
		$anr =  $entry->get_value('employeeNumber');
	}

	return $anr;
}
